#include <algorithm>

namespace Spheral {

//------------------------------------------------------------------------------
// Constructors.
//------------------------------------------------------------------------------
template<typename Dimension>
inline
UpdatePolicyBase<Dimension>::
UpdatePolicyBase(std::initializer_list<std::string> depends):
  mDependencies(depends) {
  std::sort(mDependencies.begin(), mDependencies.end());
}

//------------------------------------------------------------------------------
// The != operator.
//------------------------------------------------------------------------------
template<typename Dimension>
inline
bool
UpdatePolicyBase<Dimension>::
operator!=(const UpdatePolicyBase& rhs) const {
  return !(*this == rhs);
}

//------------------------------------------------------------------------------
// Is this state dependent?
//------------------------------------------------------------------------------
template<typename Dimension>
inline
bool
UpdatePolicyBase<Dimension>::
independent() const {
  return mDependencies.size() == 0;
}

template<typename Dimension>
inline
bool
UpdatePolicyBase<Dimension>::
dependent() const {
  return !(this->independent());
}

//------------------------------------------------------------------------------
// The set of field names this state is dependent on.
//------------------------------------------------------------------------------
template<typename Dimension>
inline
const std::vector<std::string>&
UpdatePolicyBase<Dimension>::
dependencies() const {
  return mDependencies;
}

//------------------------------------------------------------------------------
// Allow the addition of dependencies.
//------------------------------------------------------------------------------
template<typename Dimension>
inline
void
UpdatePolicyBase<Dimension>::
addDependency(const std::string& depend) {
  mDependencies.push_back(depend);
  std::sort(mDependencies.begin(), mDependencies.end());
}

}
